/**
 * Program: MyToolBar.java
 * Comments: A frame to diplay the tool bar menu.
 */

package WebFlow.ToolBar;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import WebFlow.*;
import org.omg.CORBA.*;
import WebFlow.submitJob.*;
import WebFlow.SerialHash.*;
import WebFlow.Control.*;
import WebFlow.RemoteFile.*;
import WebFlow.hashtable.*;
import WebFlow.ContextManager.*;
import java.io.File;

public class MyToolBar extends JFrame 
    implements ActionListener,MenuListener{

    JMenuBar menuBar;
    JMenu FileMenu,EditMenu,BrowserMenu,JobMenu,MonitorMenu,FBMenu,LocMenu;
    CmdMenuItem ExitItem,AddApp,RmApp,WBOpen,WBClose,
	JMOpen,JMClose,FBOpen,FBClose;
    AppletFrame fbApplet, jmApplet;
    Vector sjMenuVec,sjCmdVec;
    GOWControl gowcontrol;

    String cmd;
    String descDir;
    
    String[] probNames=null;
    String[] probVals=null;
    String userName;
    
    //The following is temporary code until I can revise.
    ORB orb;
    WebFlowContext master=null,myContext;
    org.omg.CORBA.Object myContextObj,mySubmitObj,mySerHashObj;
    org.omg.CORBA.Object myCMObj,myCDObj,mySCObj,myRFObj;
    submitJob sj;
    SerialHash sh;
    ContextManager cm;
    RemoteFile rf;
    ContextData cd;
    SaveContext sc;

    public MyToolBar(){
    }

    public MyToolBar(GOWControl gc) {
	setTitle("GOW Tool Bar");
	gowcontrol=gc;

  	menuBar=new JMenuBar();
  	setJMenuBar(menuBar);
	sjMenuVec=new Vector();
	sjCmdVec=new Vector();

	//Use GOWControl to set some things.
	descDir=gowcontrol.getDescDir();
	userName=gowcontrol.getUserName();
	cmd=gowcontrol.getBrowserCommand();
	master=gowcontrol.getMaster();

	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e){
		    System.exit(0);
		}
	    });

	try {
	    myContextObj=master.addNewContext(userName);
	    myContext=WebFlowContextHelper.narrow(myContextObj);

	    mySubmitObj=myContext.addNewModule("submit");
	    mySerHashObj=myContext.addNewModule("SerialHash");
	    myCMObj=myContext.addNewModule("ContextManager");
	    myRFObj=myContext.addNewModule("RemoteFile");
	    myCDObj=myContext.addNewModule("ContextData");
	    mySCObj=myContext.addNewModule("SaveContext");
	    sj=submitJobHelper.narrow(mySubmitObj);
	    sh=SerialHashHelper.narrow(mySerHashObj);
	    cm=ContextManagerHelper.narrow(myCMObj);
	    rf=RemoteFileHelper.narrow(myRFObj);
	    cd=ContextDataHelper.narrow(myCDObj);
	    sc=SaveContextHelper.narrow(mySCObj);
	    
	    //The following sets up the context manager and restores
	    //the context.
	    cm.init(myContext,mySCObj,myRFObj);
	    if(!cm.setUserContextData(myContext,descDir,userName)){
		sc.readContextData(myContext,descDir+File.separator+userName);
		WebFlow.ClientUtils.restoreChildren(myContext,cm,sc);
	    }
	}
	
	catch (Exception ex) {
	    ex.printStackTrace();
	}

	fbApplet=new AppletFrame(new UDapplet(),"File Browser");
	fbApplet.hide();

	//The job monitor needs a reference to the submitJob module.
	jmApplet=new AppletFrame(new TrackerApplet(this),"Job Monitor");
	jmApplet.hide();

	/* The remainder of the constructor sets up the GUI */
	FileMenu=new JMenu("File");
	ExitItem=new CmdMenuItem("Exit");
	ExitItem.addActionListener(this);
	ExitItem.setCommand(new ExitCmd());
	FileMenu.add(ExitItem);
	menuBar.add(FileMenu);

	EditMenu=new JMenu("Edit");
	AddApp=new CmdMenuItem("Add Local Application");
	AddApp.addActionListener(this);
	AddApp.setCommand(new AddCmd());
	AddApp.setEnabled(false);
	RmApp=new CmdMenuItem("Remove Local Application");
	RmApp.addActionListener(this);
	RmApp.setCommand(new RmCmd());
	RmApp.setEnabled(false);
	EditMenu.add(AddApp);
	EditMenu.add(RmApp);
	menuBar.add(EditMenu);

	BrowserMenu=new JMenu("Web Browser");
	WBOpen=new CmdMenuItem("Open");
	WBOpen.addActionListener(this);
	WBOpen.setCommand(new WBOpenCmd(cmd));
	BrowserMenu.add(WBOpen);
	menuBar.add(BrowserMenu);

	JobMenu=new JMenu("Submit Job");
	JobMenu.add("Pending Jobs:");
	JobMenu.addSeparator();
	JobMenu.addMenuListener(this);
	menuBar.add(JobMenu);

	MonitorMenu=new JMenu("Job Monitor");
	JMOpen=new CmdMenuItem("Open");
	JMOpen.addActionListener(this);
	JMOpen.setCommand(new AppOpenCmd(jmApplet));
	JMClose=new CmdMenuItem("Close");
	JMClose.addActionListener(this);
	JMClose.setCommand(new AppCloseCmd(jmApplet));
	MonitorMenu.add(JMOpen);
	MonitorMenu.add(JMClose);
	menuBar.add(MonitorMenu);
	
	FBMenu=new JMenu("File Browser");
	FBOpen=new CmdMenuItem("Open");
	FBOpen.addActionListener(this);
	FBOpen.setCommand(new AppOpenCmd(fbApplet));
	FBClose=new CmdMenuItem("Close");
	FBClose.addActionListener(this);
	FBClose.setCommand(new AppCloseCmd(fbApplet));
	FBMenu.add(FBOpen);
	FBMenu.add(FBClose);
	menuBar.add(FBMenu);

	LocMenu=new JMenu("Local Applications");
	menuBar.add(LocMenu);

    }
    
    //I implemented the "command" design pattern for the commands so that
    //they could be separated from the code that sets up the visual 
    //components.
    public void actionPerformed(ActionEvent e){
	CommandHolder ch=(CommandHolder)e.getSource();
	ch.getCommand().Execute();
    }

    //Must override the following when implementing the MenuListener.
    public void menuCanceled(MenuEvent me){
	System.out.println("Menu Canceled");
	JMenu menuSource=(JMenu)me.getSource();
	destroyDynMenu(menuSource);
    }
    
    //This is called whenever the mouse moves away from the selected
    //menu, or an item from the menu is selected and the pop-up is
    //dismissed.
    public void menuDeselected(MenuEvent me) {
	JMenu menuSource=(JMenu)me.getSource();
	destroyDynMenu(menuSource);
    }
    
    public void menuSelected(MenuEvent me) {
	JMenu menuSource=(JMenu)me.getSource();
	sh.openHash(descDir+userName+"/probHash");
	
	probNames=sh.getHashNames();
	probVals=sh.getHashValues();
	constructDynMenu(menuSource);
	sh.closeHash();
    }

    /**
     * Create the submit job menu dynamically from the static hash
     * table.
     */
    private void constructDynMenu(JMenu parent){
	for(int i=0;i<probNames.length;i++){
	    sjMenuVec.add(new CmdMenuItem(probNames[i]));
	    String dir=descDir+userName+"/probHash";
	    sjCmdVec.add(new SJCmd(probNames[i],probVals[i],sj,sh,dir));
	    ((CmdMenuItem)sjMenuVec.get(i)).
		addActionListener(this);
	    ((CmdMenuItem)sjMenuVec.get(i)).
		setCommand((SJCmd)sjCmdVec.get(i));
	    parent.add((CmdMenuItem)sjMenuVec.get(i));
	}
    }

    /**
     * The mouse has been moved away from the submit job menu, or else
     * one of the items has been selected.  Dispose of everything.
     * Note this approach relies on the garbage collector to clean up,
     * which I may regret one day.
     */
    private void destroyDynMenu(JMenu parent) {
  	for(int i=0;i<sjMenuVec.size();i++) {
  	    parent.remove((CmdMenuItem)sjMenuVec.get(i));
  	}
	sjMenuVec.clear();
	sjCmdVec.clear();
    }

    /**
     * These are some accessor methods for sharing data with other
     * objects.
     */
    
    public submitJob getSubmitJob() { return sj; }
    public ContextManager getContextManager() { return cm; }
}

	
