/**
 * program: TrackPanel.java
 * Display panel for showing queue status.
 * Calls a servlet that looks in the user's Descriptor directory for 
 * his/her jobHash file.
 *
 * We have a mini-protocal for interacting with the servlet:
 * + event=query displays the contents of jobHash.
 * + event=delete deletes the selected file from jobHash.
 *
 * Locally, the info in the jobHash file is put into a Vector so that I can
 * make use of the JList class.
 * 
 */

package WebFlow.ToolBar;

import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.net.*;
import java.io.*;
import java.util.*;
import WebFlow.ContextManager.*;

public class TrackPanel extends JPanel 
    implements ActionListener {
    
    StringBuffer responseBuffer; //holds the data read by the servlet.
    JPanel buttonPanel,queuePanel;
    JScrollPane jobPane;
    JButton viewstat,update,delete;
    JTextArea textArea,queueText;
    Hashtable paramHash;
    Vector jobVector;
    JList jobList;
    Dimension d;
    JFrame dialogFrame=new JFrame();
    String message1="Job must be finished to be deleted.";
    String message2="Please use mouse to select a job to delete.";
    TrackerApplet applet;
    String servletURL;
    PrintWriter logOut;
    ContextManager cm;

    /**------------------------------------------------------------
     * Constructor
     *------------------------------------------------------------*/
    public TrackPanel(TrackerApplet applet) {
	
	this.applet=applet;
	cm=applet.getContextManager();

	//Set the servletURL string.
	URL dBase=applet.getDocumentBase();
	String protocol,host,port;
	if(dBase!=null) {
	    protocol=dBase.getProtocol();
	    host=dBase.getHost();
	    port=(new Integer(dBase.getPort())).toString();
	}
	else {
	    //Hard-coded.  Use this if running Tracker applet inside a
	    //java application.
	    protocol="http";
	    host="dev-spt01.asc.hpc.mil";
	    port="8080";
	}
	
	//Make a call to the MonitorJobServ.
	servletURL=protocol+"://"+host+":"+port+"/GOW"
	    + "/servlet/WebFlow.Servlets.MonitorJobServ";
	
	paramHash=new Hashtable();
	jobVector=new Vector();
	setLayout(new java.awt.BorderLayout());

	//Do this now to set up the jobVector.
 	paramHash.put("event","query");
	callServlet12(servletURL);
	
	//Set up the buttons
	buttonPanel=new JPanel();
	update=new JButton("Update");
	update.addActionListener(this);
	viewstat=new JButton("Status");
	viewstat.addActionListener(this);
	delete=new JButton("Delete");
	delete.addActionListener(this);
	buttonPanel.add(update);
	buttonPanel.add(viewstat);
	buttonPanel.add(delete);

	//Set up the middle panel that displays jobs
	jobList=new JList(jobVector);
	jobList.setVisibleRowCount(10);
	jobPane=new JScrollPane(jobList,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	jobPane.setPreferredSize(new Dimension(50,150));
  	add(buttonPanel,"North");
	add(jobPane,"Center");

	//Set up lower panel that displays queue information.
	queuePanel=new JPanel();
	String initMsg="Click on a running job to obtain queue info.";
	queueText=new JTextArea(initMsg);
	JScrollPane jsp=new JScrollPane(queueText);
	jsp.setPreferredSize(new Dimension(250,50));
	queuePanel.add(jsp);
	add(queuePanel,"South");
    }

    
    /**------------------------------------------------------------
     * Action performed definitions.  If the MonitorJobServ must 
     * be called, we place the parameter name/value pairs in the
     * paramHash hashtable.
     *------------------------------------------------------------*/
    public void actionPerformed(ActionEvent ev) {
	Object source=ev.getSource();

	if(source==update) {
	    clearArea();
	    paramHash.put("event","query");
	    callServlet12(servletURL);
	    jobList.setListData(jobVector);
	}
  	else if(source==delete) {
	    paramHash.put("event","cleanup");
	    if(jobList.getSelectedValue()!=null) {
		String[] response=getJobNameAndStatus(jobList);
		if(response[1].equals("finished")){
		    paramHash.put("job",response[0].trim());		    
		    jobVector.removeAllElements();
		    callServlet12(servletURL);
		    jobList.setListData(jobVector);
		}

		else {
		    JOptionPane.showMessageDialog(dialogFrame,message1);
		}
	    }
	    else {
		JOptionPane.showMessageDialog(dialogFrame,message2);
	    }
			
	}

	//All of the following needs to be gleaned from the context
	//data somehow.
	else if(source==viewstat) {
	    String[] jobName=getJobNameAndStatus(jobList);
	    queueText.selectAll();
	    queueText.cut();
	    if(jobList.getSelectedValue()==null)
		queueText.setText("Please use mouse to select a job");

	    else if(jobName[1].equals("finished")) {
		String msg="Job is complete.  Use \"Delete\" to remove.";
		queueText.setText(msg);	    
	    }
	    else if(jobName[1]!=null){
		String command="/pbs_bin/qstat "+jobName[1];
		
		cm.setCurrentProblem(jobName[0]);
		String session=cm.getCurrentProblemProperty("CurrentChild");
		if(session.endsWith(".SC"))
		   session=session.substring(0,session.length()-3);
		cm.selectSession(jobName[0],session);

		String machine=cm.getCurrentSessionProperty("MachineName");
  		String text=(applet.getSubmitJob()).
  		    execRemoteCommand(machine,"gateway",command,"rsh"); 
		if(text!=null) 
		    queueText.setText(text);
		else
		    queueText.setText("Error: couldn't reach queue");
	    }
	    else 
		queueText.setText("No jobs submitted");
	}
    }

    /**-----------------------------------------------------------
     * Returns the name and status of the selected entry.
     *-----------------------------------------------------------*/
    private String[] getJobNameAndStatus(JList aList){
	String[] nv=new String[2];
	StringTokenizer st;
	String tmp=(String)aList.getSelectedValue();
	if(!(tmp==null)){
	    st=new StringTokenizer(tmp);
	    nv[0]=st.nextToken();
	    nv[1]=st.nextToken();
	}
	return nv;
    }

    /**-----------------------------------------------------------
     * Clear the text area.  The clear() method only works with 
     * java 1.2 VMs, so I'm using removeAllElements() so that this
     * is compatible with Netscape 4's VM.
     *------------------------------------------------------------*/
    private void clearArea() {
	System.out.println(jobVector.size());
	jobVector.removeAllElements();
    }
    
    /**------------------------------------------------------------
     * Call the servlet that reads the hashtable file.
     * Output is returned line by line and sent to 
     * the method displayResponse() for displaying in the text area.
     *------------------------------------------------------------*/
    private void callservlet(String servletURL) {
	String request=servletURL;
	String response=null;
	java.net.URL url=null;
	InputStream is=null;
	InputStreamReader isr=null;
	BufferedReader in=null;
	try {
	    // Construct the servlet string.
	    Enumeration en=paramHash.keys();
	    
	    while(en.hasMoreElements()) {
		String name=(String)en.nextElement();
		String value=(String)paramHash.get(name);
		String ch;
		if(en.hasMoreElements()) ch="&";
		else ch="";
		request+="?"+name+"="+value+ch;
		System.out.println(request);
	    }
	    //Clear out all entries.
	    paramHash.clear();
	    
	    url = new java.net.URL(request);
	    is = url.openStream();
	    isr = new InputStreamReader(is);
	    in = new BufferedReader(isr);
	    String inputLine; 
	    
	    while ((inputLine = in.readLine()) != null) {
		jobVector.addElement(inputLine);
	    }
	    in.close(); 
	}
	catch (Exception e) { 
	    System.err.println("Exception " + e);
	    response = "failure"; 
	} 
	//System.out.println("return: "+response);
    }
    /**------------------------------------------------------------
     * Call the servlet that reads the hashtable file.
     * Output is returned line by line and sent to 
     * the method displayResponse() for displaying in the text area.
     * 
     * The following version apparently does not work with 
     * Netscape.
     *------------------------------------------------------------*/
    private void callServlet12(String servletURL){
	//Based on example 3.8 from Cay Horstmann's "Core Java 2".
	//	StringBuffer response=new StringBuffer();

	java.net.URL url=null;
	java.net.URLConnection conn=null;
	BufferedReader in=null;

	try{
	    url=new java.net.URL(servletURL);
	    conn=url.openConnection();
	    conn.setDoOutput(true);
	    conn.setDoInput(true);
	    conn.setDefaultUseCaches(false);
	    conn.setUseCaches(false);
	    
  	    PrintWriter out=
  		new PrintWriter(conn.getOutputStream());

	    //The following are parameters passed to the servlet.
	    Enumeration en=paramHash.keys();
	    System.out.println("Size is "+paramHash.size());
	    while(en.hasMoreElements()) {
		String name=(String)en.nextElement();
		String value=(String)paramHash.get(name);
		String ch;
		if(en.hasMoreElements()) ch="&";
		else ch="";
		System.out.println(name+"="+URLEncoder.encode(value)+ch);
		out.print(name+"="+URLEncoder.encode(value)+ch);

	    }
	    //Clear out all entries.
	    paramHash.clear();
	    out.close();
	}
	catch (Exception e){e.printStackTrace();}

	try {
	    in=	new BufferedReader(new 
		InputStreamReader(conn.getInputStream()));

	    //Put contents of jobHash into a vector.
	    String line;
	    while((line=in.readLine()) !=null){
		jobVector.addElement(line);
	    }
	    in.close();
	    
	}
	catch(FileNotFoundException ex) {
	    InputStream err=((HttpURLConnection)conn).getErrorStream();
	    in=new BufferedReader(new InputStreamReader(err));
	}
	catch(Exception e) {e.printStackTrace();}

    }
  	
}








