/**
 * Program: TreeFactory.java
 * Comments: An abstract class that defines some methods needed to construct
 * trees from either local or remote file directories. 
 */

package WebFlow.ToolBar;

import javax.swing.tree.*;
import javax.swing.*;
import java.io.*;

public abstract class TreeFactory {
    DefaultMutableTreeNode rootNode;
    DefaultTreeModel treeModel;

    //An empty constructor.
    public TreeFactory(){ ; }
    
    /**
     * This method sets up a very simple tree.  
     */
    public JTree createTree(String dirName){
	JTree tree=null;
	rootNode=new DefaultMutableTreeNode(dirName);
	treeModel=new DefaultTreeModel(rootNode);
	tree=new JTree(treeModel);
	tree.setShowsRootHandles(true);
	String[] childFiles=returnFiles(dirName);
	addNodes(rootNode,treeModel,childFiles,dirName+File.separator);
	return tree;
    }

    /**
     * This method inserts a child tree expansion into the root.
     * We must obtain a reference to the node from the treepath.  Then
     * we clean up the "<Empty>" placeholder.
     */
    public void expandChild(TreePath treepath) {
	DefaultMutableTreeNode node,parent;
	String pathElem="";
  	String dirPath=assembleDir(treepath);
  	node=(DefaultMutableTreeNode)treepath.getLastPathComponent();
	//Clean out the placeholder.
	node.removeAllChildren();
    	String[] childFiles=returnFiles(dirPath);
    	addNodes(node,treeModel,childFiles,dirPath);
	
	if(treepath.getParentPath()!=null) {
	    String parentPath=assembleDir(treepath.getParentPath());  	
	    parent=new DefaultMutableTreeNode(parentPath);	
	    parent.add(node);
	}
    }    
    
    /**
     * Assemble a directory path from a TreePath.
     */
    public String assembleDir(TreePath treepath) {
	//Construct the path from the treepath elements.
	String dirPath="";
	String pathElem="";
	if(treepath!=null) {
	    for(int i=0;i<treepath.getPathCount();i++) {
		pathElem=treepath.getPathComponent(i).toString();
		if(!pathElem.endsWith(File.separator))
		    pathElem+=File.separator;
		dirPath+=pathElem;
	    }
	}
	return dirPath;
    }

    /**
     * Returns the last path component of the tree path.  Typically,
     * this is will be the name of a selected file.
     */
    public String getFileName(TreePath tp){
	String holder="";
	if(tp!=null) {
	    holder=(tp.getLastPathComponent()).toString();
	    System.out.println(holder);
	    return (holder);	
	}
	else {
	    System.out.println("TreePath is null!");
	    return null;
	}
    }


    /**
     * Returns the names of the files/directories immediately beneath the
     * root directory. This method is abstract because the files may 
     * be either local or remote.
     */
    public abstract String[] returnFiles(String dirName);
    
    /**
     * Add nodes to the tree.  This must be abstract because we must
     * distinguish between local and remote files.  Note that some of
     * this backbending is because of the crappy Netscape VM's lack of 
     * support for Java 1.2+ classes.
     */
    public abstract void addNodes(DefaultMutableTreeNode parent, 
				  DefaultTreeModel model, 
				  String[] fileList,String parentDir);
    
}




